@@ -41,6 +41,8 @@ ARGS_DESC = """
   |``scale-factor``   | 1.0 | scale result to watts (RMS voltage)
   |``merge``          | yes | [yes|no] use a single output stream for all phases
   |``polar``          | no  | [yes|no] use polar coordinates (mag, angle) or cartesian (P,Q)
+  |``line-freq``      | 60  | line frequency
+  |``samp-freq``      | --  | sampling frequency
 
 :inputs:
  
@@ -195,6 +197,12 @@ class Prep(FilterModule):
                             help="use a single output stream for all phases")
         parser.add_argument("--polar", type=utilities.yesno, default=False,
                             help="use polar coordinates (mag, angle) or cartesian (P,Q)")
+        parser.add_argument("--line-freq", type=int, default=60,
+                            help="sampling frequency in Hz")
+        parser.add_argument("--samp-freq", type=int, required=True,
+                            help="sampling frequency in Hz")
+        parser.add_argument("--goertzel", type=utilities.yesno,
+                            help="use goertzel algorithm or fft")
         parser.description = ARGS_DESC
 
     n_output = 0
@@ -237,6 +245,9 @@ class Prep(FilterModule):
             rows = iv_data.shape[0]
             data_timestamps = iv_data[:, 0]
             zc_timestamps = zc_data[:, 0]
+            samp_freq = parsed_args.samp_freq
+            line_freq = parsed_args.line_freq
+
             # check to make sure the the zc data overlaps with iv data
             # if we lose a voltage signal iv stalls looking for zc data
             # flush the old iv data because there are not any zc's for it
@@ -272,7 +283,7 @@ class Prep(FilterModule):
             out = np.zeros((1, nphases * (nharm * 2) + 1))
             # Pull out sinefit data for the entire time range of this block
             for sinefit_line in zc_data:
-                async def prep_period(t_min, t_max, angle_shift):
+                async def prep_period(t_min, t_max, angle_shift,f_expected,f_samp):
                     """
                     Compute prep coefficients from time t_min to t_max, which
                     are the timestamps of the start and end of one period.
@@ -293,11 +304,18 @@ class Prep(FilterModule):
 
                     # Perform FFT over those indices
                     N = idx_max - idx_min
+                    freq_vect = np.zeros(nharm)
                     for ph in range(nphases):
                         col = parsed_args.current_indices[ph]
                         rot = parsed_args.rotations[ph] - angle_shift
                         d = iv_data[idx_min:idx_max, col]
-                        F = scipy.fftpack.fft(d) * 2.0 / N
+                        
+                        if parsed_args.goertzel:
+                            for k in range(nharm):
+                                freq_vect[k] = (2*k+1)*f_expected
+                            F = helpers.goertzel(d,f_samp,freq_vect)
+                        else:
+                            F = scipy.fftpack.fft(d) * 2.0 / N
 
                         # If we wanted more harmonics than the FFT gave us, pad
                         # with zeros
@@ -307,8 +325,11 @@ class Prep(FilterModule):
                         # Fill output data.
                         out[0, 0] = round(t_min)
                         for k in range(nharm):
-                            Fk = F[2 * k + 1] * \
-                                 np.e ** (rot * 1j * (2 * k + 1)) / lag_correction
+                            if parsed_args.goertzel:
+                                Fk = F[k]*np.e**(rot * 1j * (2 * k + 1)) / lag_correction
+                            else:
+                                Fk = F[2 * k + 1] * \
+                                    np.e**(rot * 1j * (2 * k + 1)) / lag_correction
                             # compute output values
                             if parsed_args.polar:
                                 out_a = np.abs(Fk) * scale_factor  # VA
@@ -321,7 +342,7 @@ class Prep(FilterModule):
                             out[0, ph * nharm * 2 + 2 * k + 2] = out_b
 
                     await insert_if_nonoverlapping(out)
-                    zc.consume(1)
+                    # zc.consume(1)
                     return idx_max
 
                 # Extract sinefit data to get zero crossing timestamps.
@@ -343,11 +364,14 @@ class Prep(FilterModule):
                     # Run prep computation
                     idx_max = await prep_period(shifted_min,
                                                 shifted_max,
-                                                angle_shift)
+                                                angle_shift, line_freq, samp_freq)
                     if not idx_max:
                         break
                     processed = idx_max
 
+                    if n == (nshift - 1):
+                        zc.consume(1)
+
             """
             # If we processed no data but there's lots in here, pretend we
             # processed half of it.
