@@ -228,7 +228,8 @@ class RawToPrep(CompositeModule):
                                                           parsed_args.min_voltage),
             'prep':           self._build_prep_configs(my_meter,
                                                        parsed_args.merge_prep,
-                                                       parsed_args.polar)
+                                                       parsed_args.polar,
+                                                       parsed_args.frequency)
         }
 
         out_iv = outputs['iv']
@@ -331,7 +332,7 @@ class RawToPrep(CompositeModule):
             'min_amp': amp,
         }
 
-    def _build_prep_configs(self, meter, merge_output, polar):
+    def _build_prep_configs(self, meter, merge_output, polar, freq):
 
         def _compute_rotations(meter):
             if(meter['type'] == 'noncontact'):
@@ -344,6 +345,12 @@ class RawToPrep(CompositeModule):
                 # convert to radians
                 return [x*(2*np.pi)/360.0 for x in rotations]
 
+        def _compute_sampling_frequency(meter):
+            if(meter['type'] == 'noncontact'):
+                return 3000
+            else:  # contact meter
+                return 8000
+
         def _compute_current_indices(meter):
             if(meter['type'] == 'noncontact'):
                 # [ts, V, --> I1, I2, I3]
@@ -355,15 +362,26 @@ class RawToPrep(CompositeModule):
         def _compute_scale_factor(meter):
             line_rms = meter['sensors']['voltage']['nominal_rms_voltage']
             return line_rms / np.sqrt(2)  # convert amps to rms
+
+        def _compute_nshift(meter):
+            nshift = meter['streams']['prep']['nshift']
+            return nshift
+
+        def _compute_goertzel(meter):
+            goertzel = meter['streams']['prep']['goertzel']
+            return goertzel
             
         return {
-            'nshift': 1,
+            'nshift': _compute_nshift(meter),
             'nharm': 4,
             'current_indices': _compute_current_indices(meter),
             'rotations': _compute_rotations(meter),
             'scale_factor': _compute_scale_factor(meter),
             'merge': merge_output,
-            'polar': polar
+            'polar': polar,
+            'line_freq': freq,
+            'samp_freq': _compute_sampling_frequency(meter),
+            'goertzel': _compute_goertzel(meter)
         }
 
     
